// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*****************************************************************************
 *
 *  FileLocator.h                         XMM Science Analysis System (SAS)
 *
 *  (c) 1997 European Space Agency  -  Written by Marco Beijersbergen
 *
 *****************************************************************************/

#ifndef __FileLocator_h
#define __FileLocator_h

#include <string>
#include <list>
#include <cstdlib>

// Searches for a file in a data search path

class FileLocator {

  public:

    FileLocator();

    // Construct from a colon-separated list of directories
    // (or optionally any other separation character).
    FileLocator(const std::string& dirList, char separator=':');

    FileLocator(const std::list<std::string>& directories);

    // Add a colon separated list of directories to the search path.
    void add(const std::string& directories, char separator=':');

    void add(const std::list<std::string>& directories);

    // Returns the full path name of a file if found in the search path.
    // The behaviour in case of an error (invalid file name, file not found)
    // depends on generateError/generateWarning. If generateError is true,
	// an error is raised, otherwise if generateWarning is true only a message
    // is generated and the function returns an empty string.
    // If the failMessage is specified, it is included in the error message.
    std::string locate(const std::string& fileName,
					   const std::string& failMessage="",
	bool generateError=true, bool generateWarning=false) const;

    friend std::ostream& operator<<(std::ostream&, const FileLocator&);

  private:

    std::list<std::string> _directories;
};

#endif
