// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	SimpleScanner.h	--- defines the SimpleScanner class which represents
//                          a simple scanner.
//
//	DESCRIPTION:
//
//	USAGE EXAMPLES:
//
//	CHANGE HISTORY:
//
//	AUTHOR:
//		M.Thomas
//
//	REFERENCES:
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/utils/src/SimpleScanner.h,v 1.4 2008/09/26 16:38:55 eojero Exp $
///

#ifndef SIMPLE_SCANNER_H
#define SIMPLE_SCANNER_H


#include "SimpleToken.h"


class SimpleScanner
{
	public :

		SimpleScanner( const std::string& s, const std::string& separators );
		~SimpleScanner();

		class Expected {
		public :
			Expected( const std::string& message );
			std::string message() const;
		private :
			std::string	_message;
		};

		bool 		eof() const;
		const 		SimpleToken& next() const;
		bool	 	nextIsCharacter( char thisCharacter ) const;
		bool 		nextIsString() const;
		SimpleToken accept();
		void 		skip();
		SimpleToken match( SimpleToken::Id id ) throw( Expected );
		SimpleToken match( char c ) throw( Expected );
		std::string	remainder() const;

	private :

		SimpleToken		_lookahead;
		void			fetchNextToken();
		std::string		_source;
		std::istream&	_ifs;
		std::string		_separators;
};

#endif

