// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// Copyright 1997 (c) European Space Agency - written by Marco Beijersbergen

#include "Tracer.h"
#include <sstream>

Tracer tracer(std::cerr);

Tracer::Tracer(std::ostream& os) : _os(os), _verbosity(Silent), _indent(0)
{

  // Determine verbosity level from the environment variable SAS_TRACER
  char *verboseFlag = getenv("SAS_TRACER");
  if (verboseFlag) {
    std::stringstream ss;
    ss << verboseFlag;
    int i;
    ss >> i;
    if (!ss)
      _verbosity = Verbose;
    else {
      if (i <= 0)
	_verbosity = Silent;
      else if (i == 1)
	_verbosity = Stack;
      else if (i == 2)
        _verbosity = Sparse;
      else
	_verbosity = Verbose;
    }
  }
}

Tracer::~Tracer()
{
  if (!_stack.empty()) {
    _os << "Stack:" << std::endl;
    for (std::list<std::string>::const_iterator i = _stack.begin();
		i != _stack.end(); ++i)
      _os << (*i) << std::endl;
  }
}

void Tracer::in(const std::string& mess)
{
   _stack.push_back(indent() + mess);
   if (_verbosity >= Verbose) {
     _os << indent() << mess << std::endl;
     ++_indent;
   }
}

void Tracer::out()
{
  if (_stack.empty()) {
    _os << "error in Tracer: more out than in"<<std::endl;
    return;
  }
  if (_verbosity >= Verbose) {
    if (_indent > 0) --_indent;
    else _os << "error in indent" << std::endl;
    _os << indent() << "done " << _stack.back() << std::endl;
  }
  _stack.pop_back();
}













