// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	stime.cc --- implementation of C I/F layer to STime class
//
//	DESCRIPTION:
//		See STime.h
//
//	CHANGE HISTORY:
//		UL, 2001-05-10: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers,	Thu May 10 14:10:42 MET DST 2001
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/utils/src/stime_c.cc,v 1.6 2008/09/26 16:38:55 eojero Exp $
//

#include <cstring>

#include "STime.h"
#include "stime_c.h"
#include "f90cxx.h"
#include "Handle.h"


#define STC(ptr)	((STime *)ptr)
#define CSTC(ptr)	((const STime *)ptr)
#define JDC(ptr)	((STime::JulianDayNumber *)ptr)
#define CJDC(ptr)	((const STime::JulianDayNumber *)ptr)

typedef Handle<STime>	STimeHandleF90;
typedef Handle<STime::JulianDayNumber>	JDNumberHandleF90;

extern "C" {

void initSTime() {}

STimeHandle ST_new0()
{
	return new STime();
}

STimeHandleF90 CXXFUNC(st_new0, ST_new0)()
{
	return STimeHandleF90(STC(ST_new0()));
}

STimeHandle ST_new1(const char *stimeStr)
{
	return new STime(stimeStr);
}

STimeHandleF90 CXXFUNC(st_new1, ST_new1)(const char *stimeStr, Integer3 len)
{
	std::string s(trim(std::string(stimeStr, len)));
	return STimeHandleF90(STC(ST_new1(s.c_str())));
}

int ST_isValidTime1(const char *stimeStr)
{
	return STime::isValidTime(stimeStr);
}

Logical CXXFUNC(st_isvalidtime1, ST_isValidTime1)(const char *stimeStr,
												  Integer3 len)
{
	std::string s(trim(std::string(stimeStr, len)));
	return ST_isValidTime1(s.c_str());
}

STimeHandle ST_new2(unsigned year, unsigned month,
					unsigned day, unsigned hour,
					unsigned minute, unsigned secs,
					double excessSecs)
{
	return new STime(year, month, day, hour, minute, secs, excessSecs);
}


STimeHandleF90 CXXFUNC(st_new2, ST_new2)
				(const Integer3 *year, const Integer3 *month,
				 const Integer3 *day, const Integer3 *hour,
				 const Integer3 *minute, const Integer3 *secs,
				 const Double *excessSecs)
{
	return STimeHandleF90(STC(ST_new2(*year,
									   month ? *month : 1,
									   day ? *day : 1,
									   hour ? *hour : 0,
									   minute ? *minute : 0,
									   secs ? *secs : 0,
									   excessSecs ? *excessSecs : 0.)));
}

int	ST_isValidTime2(unsigned year, unsigned month,
					unsigned day, unsigned hour,
					unsigned minute, unsigned secs,
					double excessSecs)
{
	return STime::isValidTime(year, month, day, hour, minute, secs,excessSecs);
}

Logical	CXXFUNC(st_isvalidtime2, ST_isValidTime2)
				(const Integer3 *year, const Integer3 *month,
				 const Integer3 *day, const Integer3 *hour,
				 const Integer3 *minute, const Integer3 *secs,
				 const Double *excessSecs)
{
	return ST_isValidTime2(*year,
						   month ? *month : 1,
						   day ? *day : 1,
						   hour ? *hour : 0,
						   minute ? *minute : 0,
						   secs ? *secs : 0,
						   excessSecs ? *excessSecs : 0.);
}

STimeHandle ST_new3(double timeVal, ST_Representation r)
{
	return new STime(timeVal, STime::Representation(r));
}

STimeHandleF90 CXXFUNC(st_new3, ST_new3)
				(const Double *timeVal, const ST_Representation *r)
{
	return STimeHandleF90(STC(ST_new3(*timeVal,
									  r ? *r : MJD)));
}

int ST_isValidTime(double timeVal, ST_Representation r)
{
	return STime::isValidTime(timeVal, (STime::Representation)r);
}

Logical	CXXFUNC(st_isvalidtime3, ST_isValidTime3)
				(const Double *timeVal, const ST_Representation *r)
{
	return ST_isValidTime(*timeVal, r ? *r : MJD);
}

STimeHandle ST_new4(unsigned long timeValInt, double timeValFrac,
					ST_Representation r)
{
	return new STime(timeValInt, timeValFrac, (STime::Representation)r);

}

STimeHandleF90	CXXFUNC(st_new4, ST_new4)
				(const Integer3 *timeValInt, const Double *timeValFrac,
				 const ST_Representation *r)
{
	return STimeHandleF90(STC(ST_new4(*timeValInt,
									  timeValFrac ? *timeValFrac : 0.,
									  r ? *r : MJD)));
}

STimeHandle ST_clone(STimeHandle time)
{
	return new STime(*CSTC(time));
}

STimeHandleF90 CXXFUNC(st_clone, ST_clone)
				(const STimeHandleF90 *t)
{
	return STimeHandleF90(STC(ST_clone(t->addressOfObject())));
}

void ST_release(STimeHandle t)
{
	delete STC(t);
}

void CXXFUNC(st_releasetime, ST_releaseTime)(STimeHandleF90 *t)
{
	ST_release(t->addressOfObject());
	*t = STimeHandleF90();
}

unsigned short ST_year(STimeHandle t)
{
	return CSTC(t)->year();
}

Integer2 CXXFUNC(st_year, ST_year)(const STimeHandleF90 *t)
{
	return ST_year(t->addressOfObject());
}

unsigned short ST_month(STimeHandle t)
{
	return CSTC(t)->month();
}

Integer2 CXXFUNC(st_month, ST_month)(const STimeHandleF90 *t)
{
	return ST_month(t->addressOfObject());
}

unsigned short ST_day(STimeHandle t)
{
	return CSTC(t)->day();
}

Integer2 CXXFUNC(st_day, ST_day)(const STimeHandleF90 *t)
{
	return ST_day(t->addressOfObject());
}

unsigned short ST_hour(STimeHandle t)
{
	return CSTC(t)->hour();
}

Integer2 CXXFUNC(st_hour, ST_hour)(const STimeHandleF90 *t)
{
	return ST_hour(t->addressOfObject());
}

unsigned short ST_minute(STimeHandle t)
{
	return CSTC(t)->minute();
}

Integer2 CXXFUNC(st_minute, ST_minute)(const STimeHandleF90 *t)
{
	return ST_minute(t->addressOfObject());
}

unsigned short ST_seconds(STimeHandle t)
{
	return CSTC(t)->seconds();
}

Integer2 CXXFUNC(st_seconds, ST_seconds)(const STimeHandleF90 *t)
{
	return ST_seconds(t->addressOfObject());
}

double ST_fracSeconds(STimeHandle t)
{
	return CSTC(t)->fracSeconds();
}

unsigned short ST_weekday(STimeHandle t)
{
	return CSTC(t)->weekday();
}

Integer2 CXXFUNC(st_weekday, ST_weekday)(const STimeHandleF90 *t)
{
	return ST_weekday(t->addressOfObject());
}

unsigned short ST_leapSeconds(STimeHandle t)
{
	return CSTC(t)->leapSeconds();
}

Integer2 CXXFUNC(st_leapseconds, ST_leapSeconds)(const STimeHandleF90 *t)
{
	return ST_leapSeconds(t->addressOfObject());
}

STimeHandle ST_assign(STimeHandle lhs, STimeHandle rhs)
{
	*STC(lhs) = *CSTC(rhs);
	return lhs;
}

void CXXFUNC(st_assign, ST_assign)(STimeHandleF90 *lhs,
								   const STimeHandleF90 * rhs)
{
	ST_release(lhs->addressOfObject());
	*lhs = STimeHandleF90(STC(ST_clone(rhs->addressOfObject())));
}

STimeHandle ST_addTo(STimeHandle t, double s)
{
	*STC(t) += s;
	return t;
}

void CXXFUNC(st_addto, ST_addTo)(const STimeHandleF90 *t, const Double *s)
{
	ST_addTo(t->addressOfObject(), *s);
}

STimeHandle ST_subtractFrom(STimeHandle t, double s)
{
	*STC(t) -= s;
	return t;
}

void CXXFUNC(st_subtractfrom, ST_subtractFrom)(const STimeHandleF90 *t,
											   const Double *s)
{
	ST_subtractFrom(t->addressOfObject(), *s);
}

STimeHandle ST_currentTime(int asGMT)
{
	return new STime(STime::currentTime(asGMT));
}

STimeHandleF90 CXXFUNC(st_currenttime, ST_currentTime)(const Logical *asGMT)
{
	return STimeHandleF90(STC(ST_currentTime(asGMT ? *asGMT : 1)));
}

double ST_secondsPerMinute = STime::secondsPerMinute;
double ST_secondsPerHour = STime::secondsPerHour;
double ST_secondsPerDay = STime::secondsPerDay;

Double CXXFUNC(st_secondsperday, ST_secondsPerDay)()
{
	return ST_secondsPerDay;
}

Double CXXFUNC(st_secondsperminute, ST_secondsPerMinute)()
{
	return ST_secondsPerMinute;
}

Double CXXFUNC(st_secondsperhour, ST_secondsPerHour)()
{
	return ST_secondsPerHour;
}

STimeHandle ST_earliest = STimeHandle(&STime::earliest);
STimeHandle ST_latest = STimeHandle(&STime::latest);

STimeHandleF90 CXXFUNC(st_earliest, ST_earliest)()
{
	return STimeHandleF90(STC(ST_earliest));
}

STimeHandleF90 CXXFUNC(st_latest, ST_latest)()
{
	return STimeHandleF90(STC(ST_latest));
}

JDNumberHandle ST_newJulianDay(unsigned long day, double fractionalDay)
{
	return new STime::JulianDayNumber(day, fractionalDay);
}

JDNumberHandleF90 CXXFUNC(st_newjulianday, ST_newJulianDay)
				(const Integer3 *day, const Double *fractionalDay)
{
	return JDNumberHandleF90(JDC(ST_newJulianDay(*day,
									 fractionalDay ? *fractionalDay : 0.)));
}

unsigned long ST_julianDayDay(JDNumberHandle julday)
{
	return CJDC(julday)->day();
}

Integer3 CXXFUNC(st_juliandayday, ST_julianDayDay)
			(const JDNumberHandleF90 *julday)
{
	return ST_julianDayDay(julday->addressOfObject());
}

void ST_julianDaySetDay(JDNumberHandle julday, unsigned long daynr)
{
	JDC(julday)->day(daynr);
}

void CXXFUNC(st_juliandaysetday, ST_julianDaySetDay)
				(const JDNumberHandleF90 *julday, const Integer3 *daynr)
{
	ST_julianDaySetDay(julday->addressOfObject(), *daynr);
}

double ST_julianDayFracDay(JDNumberHandle julday)
{
	return CJDC(julday)->fractionalDay();
}

Double CXXFUNC(st_juliandayfracday, ST_julianDayFracDay)
				(const JDNumberHandleF90 *julday)
{
	return ST_julianDayFracDay(julday->addressOfObject());
}

void ST_julianDaySetFracDay(JDNumberHandle julday, double fracday)
{
	JDC(julday)->fractionalDay(fracday);
}

void CXXFUNC(st_juliandaysetfracday, ST_julianDaySetFracDay)
				(const JDNumberHandleF90 *julday, const Integer3 *daynr)
{
	ST_julianDaySetFracDay(julday->addressOfObject(), *daynr);
}

double ST_julianDayFull(JDNumberHandle julday)
{
	return CJDC(julday)->full();
}

Double CXXFUNC(st_juliandayfull, ST_julianDayFull)
											(const JDNumberHandleF90 *julday)
{
	return ST_julianDayFull(julday->addressOfObject());
}

void ST_releaseJulianDay(JDNumberHandle julday)
{
	delete JDC(julday);
}

void CXXFUNC(st_releasejulianday, ST_releaseJulianDay)
													(JDNumberHandleF90 *julday)
{
	ST_releaseJulianDay(julday->addressOfObject());
}

JDNumberHandle ST_julianDay(STimeHandle t, ST_TimeSystem timeSystem)
{
	return new STime::JulianDayNumber(CSTC(t)->julianDay(
											STime::TimeSystem(timeSystem)));
}

JDNumberHandleF90 CXXFUNC(st_julianday, ST_julianDay)
				(const STimeHandleF90 *t, const ST_TimeSystem *timeSystem)
{
	return JDNumberHandleF90(JDC(ST_julianDay(t->addressOfObject(),
							timeSystem ? *timeSystem : UTC)));
}

JDNumberHandle ST_modifiedJulianDay(STimeHandle t,
									ST_TimeSystem timeSystem)
{
	return new STime::JulianDayNumber(CSTC(t)->modifiedJulianDay(
											STime::TimeSystem(timeSystem)));
}

JDNumberHandleF90 CXXFUNC(st_modifiedjulianday, ST_modifiedJulianDay)
				(const STimeHandleF90 *t, const ST_TimeSystem *timeSystem)
{
	return JDNumberHandleF90(JDC(ST_modifiedJulianDay(t->addressOfObject(),
							timeSystem ? *timeSystem : UTC)));
}

double ST_siderealTime(STimeHandle t, double longitude)
{
	return CSTC(t)->siderealTime(longitude);
}

Double CXXFUNC(st_siderealtime, ST_siderealTime)
				(const STimeHandleF90 *t, const Double *longitude)
{
	return ST_siderealTime(t->addressOfObject(), longitude ? *longitude : 0.);
}

char *ST_toString(STimeHandle t, unsigned fracSecondDigits,
						ST_StringRepresentation form)
{
	std::string s(CSTC(t)->toString(fracSecondDigits,
									   STime::StringRepresentation(form)));
	char *p = new char[s.length()+1];
	s.copy(p, std::string::npos);
	p[s.length()] = 0;

	return p;
}

void CXXFUNC(st_tofixedstring, ST_toFixedString)
				(char *s, Integer3 length,
				 const STimeHandleF90 *t, const Integer3 *fracSecondDigits,
				 const ST_StringRepresentation *form)

{
	char *p = ST_toString(t->addressOfObject(),
						  fracSecondDigits ? *fracSecondDigits : 3,
						  form ? *form : FITS);
	unsigned len = std::strlen(p);
	std::strncpy(s, p, length);
	delete p;
	std::memset(s+len, ' ', length-len);
}

STimeHandle ST_add(STimeHandle t, double s)
{
	return new STime(*CSTC(t) + s);
}

STimeHandleF90 CXXFUNC(st_add, ST_add)(const STimeHandleF90 *t,
									   const Double *s)
{
	return STimeHandleF90(STC(ST_add(t->addressOfObject(), *s)));
}

STimeHandle ST_sub(STimeHandle t, double s)
{
	return new STime(*CSTC(t) - s);
}

STimeHandleF90 CXXFUNC(st_sub, ST_sub)(const STimeHandleF90 *t,
									   const Double *s)
{
	return STimeHandleF90(STC(ST_sub(t->addressOfObject(), *s)));
}

double ST_diff(STimeHandle t1, STimeHandle t2)
{
	return *CSTC(t1) - *CSTC(t2);
}

Double CXXFUNC(st_diff, ST_diff)(const STimeHandleF90 *t1,
								 const STimeHandleF90 *t2)
{
	return ST_diff(t1->addressOfObject(), t2->addressOfObject());
					
}

int ST_isLaterThan(STimeHandle t1, STimeHandle t2)
{
	return *CSTC(t1) > *CSTC(t2);
}

Logical	CXXFUNC(st_islaterthan, ST_isLaterThan)
				(const STimeHandleF90 *t1, const STimeHandleF90 *t2)
{
	return ST_isLaterThan(t1->addressOfObject(), t2->addressOfObject());
}

int ST_isLaterThanOrSameInstant(STimeHandle t1, STimeHandle t2)
{
	return *CSTC(t1) >= *CSTC(t2);
}

Logical	CXXFUNC(st_islaterthanorsameinstant, ST_isLaterThanOrSameInstant)
				(const STimeHandleF90 *t1, const STimeHandleF90 *t2)
{
	return ST_isLaterThanOrSameInstant(t1->addressOfObject(),
									   t2->addressOfObject());
}

int ST_isEarlierThan(STimeHandle t1, STimeHandle t2)
{
	return *CSTC(t1) < *CSTC(t2);
}

Logical	CXXFUNC(st_isearlierthan, ST_isEarlierThan)
				(const STimeHandleF90 *t1, const STimeHandleF90 *t2)
{
	return ST_isEarlierThan(t1->addressOfObject(), t2->addressOfObject());
}

int ST_isEarlierThanOrSameInstant(STimeHandle t1,
								  STimeHandle t2)
{
	return *CSTC(t1) <= *CSTC(t2);
}

Logical	CXXFUNC(st_isearlierthanorsameinstant, ST_isEarlierThanOrSameInstant)
				(const STimeHandleF90 *t1, const STimeHandleF90 *t2)
{
	return ST_isEarlierThanOrSameInstant(t1->addressOfObject(),
									     t2->addressOfObject());
}

int ST_isSameInstant(STimeHandle t1, STimeHandle t2)
{
	return *CSTC(t1) == *CSTC(t2);	
}

Logical	CXXFUNC(st_issameinstant, ST_isSameInstant)
				(const STimeHandleF90 *t1, const STimeHandleF90 *t2)
{
	return ST_isSameInstant(t1->addressOfObject(), t2->addressOfObject());
}

int ST_isNotSameInstant(STimeHandle t1, STimeHandle t2)
{
	return *CSTC(t1) != *CSTC(t2);	
}

Logical	CXXFUNC(st_isnotsameinstant, ST_isNotSameInstant)
				(const STimeHandleF90 *t1, const STimeHandleF90 *t2)
{
	return ST_isNotSameInstant(t1->addressOfObject(), t2->addressOfObject());
}

}
